using namespace std;

#include <stdlib.h>
#include "my_serial.h"

serial_port my_port;



// open() the COM port, set it up, purge it, close() it
int a_single_pass(char* dev, unsigned long baud)
{
 int retval = 0;

   if (my_port.open(dev) != 0) 
   {
      printf("failed to open() the COM port (%s).\n", dev);
      return(1);
   }

   if (my_port.cfg_baud_and_format(
                            baud,      // baud
                            8,         // data bits
                            FALSE,     // parity enable
                            NOPARITY,  // parity type
                            ONESTOPBIT // stop bits 
                                   ) != 0)
   {
      printf("failed to configure baud and format on the COM port (%s).\n", dev);
      retval = 1;
      goto modem_init_over;
   }

   // disable HW flow control
   if (my_port.cfg_hw_flowctl(
                             RTS_CONTROL_DISABLE,    // rts 
                             FALSE                   // cts
                              ) != 0)
   {
      printf("failed to configure rts/cts on the COM port (%s).\n", dev);
      retval = 1;
      goto modem_init_over;
   }

   if (my_port.cfg_sw_flowctl(
                             FALSE, // TX XON/XOFF (out)
                             FALSE  // RX XON/XOFF (in)
                              ) != 0)
   {
      printf("failed to configure XON/XOFF on the COM port (%s).\n", dev);
      retval = 1;
      goto modem_init_over;
   }

   if (my_port.cfg_callctl(
                             DTR_CONTROL_HANDSHAKE, // dtr
                             //TRUE                   // dsr
                             FALSE                   // dsr -- this modem doesn't handle DSR
                           ) != 0)
   {
      printf("failed to configure dtr/dsr on the COM port (%s).\n", dev);
      retval = 1;
      goto modem_init_over;
   }

	if (my_port.set_comm_timeouts(
                                0,   // ReadIntervalTimeout
                                0,   // ReadTotalTimeoutMultiplier
                                0,   // ReadTotalTimeoutConstant
                                0,   // WriteTotalTimeoutMultiplier
                                2000 // WriteTotalTimeoutConstant
                                 ) != 0)
   {
      printf("failed to configure timeouts on the COM port (%s).\n", dev);
      retval = 1;
      goto modem_init_over;
   }

   my_port.purge();

modem_init_over:
   my_port.close();

   return(retval);
}



int main(int argc, char** argv, char** env)
{
   if (argc <= 1)
   {
      printf("Usage: \n");
      printf("  comreset <COM port name>\n");
      exit(2);
   }

   if (a_single_pass(argv[1], 9600) != 0) exit(1);
   if (a_single_pass(argv[1], 115200) != 0) exit(1);
   if (a_single_pass(argv[1], 9600) != 0) exit(1);
   
   exit(0);
}

