using namespace std;

#include "my_serial.h"

#ifdef DEBUG
#include <stdio.h>
#include "my_err.h"
#endif

serial_port::serial_port()
{
}



serial_port::serial_port(char* portname)
{
   if (portname != NULL)
      name = portname;
#ifdef DEBUG
   else
      printf("Error: serial_port::serial_port(char* portname) obtained a NULL pointer!\n");
#endif
}



serial_port::~serial_port()
{
}



int serial_port::open(char* portname)
{
 int retval = -1;

   if (portname != NULL)
   {
      name = portname;
      retval = open();
   }
   else
   {
#ifdef DEBUG
      printf("Error: serial_port::open(char* portname) obtained a NULL pointer!\n");
#endif
   }

   return retval;
}



int serial_port::open()
{
 int retval = 0;
 string tmp_portname;

   if (name.size() != 0)
   {
      tmp_portname = "\\\\.\\" + name;

      handle = CreateFile(
                      tmp_portname.c_str(),
                      GENERIC_READ | GENERIC_WRITE,
                      0,
                      0,
                      OPEN_EXISTING,
                      0,
                      0
                        );

		if (handle == INVALID_HANDLE_VALUE)
      {
#ifdef DEBUG
         check_and_report_misc_error();
         printf("Error: in serial_port::open() : CreateFile failed\n");
#endif
         retval = -1;
      }
      // else opened fine, retval = 0
   }
   else
   {
#ifdef DEBUG
      printf("Error: in serial_port::open() : port name is an empty string! (uninitialized?)\n");
#endif
      retval = -1;
   }

   return retval;
}



void serial_port::close()
{
   CloseHandle(handle);
   return;
}



int serial_port::cfg_baud_and_format(
                  unsigned long baud, 
                  unsigned char char_len, 
                  unsigned long parity_enable, 
                  unsigned char parity_type, 
                  unsigned char stop_bits
                                    )
{
 BOOL ready;
 DCB attribs;

   attribs.DCBlength = sizeof(DCB);
   ready = GetCommState(handle, &attribs);

	if (FALSE == ready)
   {
#ifdef DEBUG
      check_and_report_misc_error();
      printf("Error: GetCommState returned a FALSE.\n");
#endif
		CloseHandle(handle);
      return 1;
   }

   attribs.BaudRate = baud;
   attribs.ByteSize = char_len;
   attribs.Parity = parity_type;
   attribs.StopBits = stop_bits;
   attribs.fBinary = TRUE;
   attribs.fParity = parity_enable;
   attribs.fNull = FALSE;
   attribs.fAbortOnError = TRUE;
   attribs.fDsrSensitivity = FALSE; // something we definitely do not need

   ready = SetCommState(handle, &attribs);
   
   if(FALSE == ready)
   {
#ifdef DEBUG
      check_and_report_misc_error();
      printf("Error: SetCommState returned a FALSE.\n");
#endif
      CloseHandle(handle);
      return 2;
   }

   return 0;
}



int serial_port::cfg_hw_flowctl(unsigned int rts, unsigned int cts)
{
 BOOL ready;
 DCB attribs;

   attribs.DCBlength = sizeof(DCB);
   ready = GetCommState(handle, &attribs);

	if (FALSE == ready)
   {
#ifdef DEBUG
      printf("Error: GetCommState returned a FALSE.\n");
#endif
		CloseHandle(handle);
      return 1;
   }

   attribs.fRtsControl = rts;
   attribs.fOutxCtsFlow = cts;

   ready = SetCommState(handle, &attribs);
   
   if(FALSE == ready)
   {
#ifdef DEBUG
      printf("Error: SetCommState returned a FALSE.\n");
#endif
      CloseHandle(handle);
      return 2;
   }

   return 0;
}



int serial_port::cfg_sw_flowctl(unsigned int x_out, unsigned int x_in)
{
 BOOL ready;
 DCB attribs;

   attribs.DCBlength = sizeof(DCB);
   ready = GetCommState(handle, &attribs);

	if (FALSE == ready)
   {
#ifdef DEBUG
      printf("Error: GetCommState returned a FALSE.\n");
#endif
		CloseHandle(handle);
      return 1;
   }

   attribs.fOutX = x_out; // TX XON/XOFF
   attribs.fInX = x_in;  // RX XON/XOFF

   ready = SetCommState(handle, &attribs);
   
   if(FALSE == ready)
   {
#ifdef DEBUG
      printf("Error: SetCommState returned a FALSE.\n");
#endif
      CloseHandle(handle);
      return 2;
   }

   return 0;
}



int serial_port::cfg_callctl(unsigned int dtr, unsigned int dsr)
{
 BOOL ready;
 DCB attribs;

   attribs.DCBlength = sizeof(DCB);
   ready = GetCommState(handle, &attribs);

	if (FALSE == ready)
   {
#ifdef DEBUG
      printf("Error: GetCommState returned a FALSE.\n");
#endif
		CloseHandle(handle);
      return 1;
   }

   attribs.fDtrControl = dtr;
   attribs.fOutxDsrFlow = dsr;

   ready = SetCommState(handle, &attribs);
   
   if(FALSE == ready)
   {
#ifdef DEBUG
      printf("Error: SetCommState returned a FALSE.\n");
#endif
      CloseHandle(handle);
      return 2;
   }

   return 0;
}



int serial_port::set_comm_timeouts(
                     unsigned long ReadIntervalTimeout,
                     unsigned long ReadTotalTimeoutMultiplier,
                     unsigned long ReadTotalTimeoutConstant,
                     unsigned long WriteTotalTimeoutMultiplier,
                     unsigned long WriteTotalTimeoutConstant
                                         )
{
 COMMTIMEOUTS timeouts;
 BOOL ready;

   ready = GetCommTimeouts(handle, &timeouts);

   if (FALSE == ready)
   {
#ifdef DEBUG
      printf("Error: GetCommTimeouts returned a FALSE.\n");
#endif
      return 1;
   }
   
   timeouts.ReadIntervalTimeout = ReadIntervalTimeout;
   timeouts.ReadTotalTimeoutConstant = ReadTotalTimeoutConstant;
   timeouts.ReadTotalTimeoutMultiplier = ReadTotalTimeoutMultiplier;
   timeouts.WriteTotalTimeoutConstant = WriteTotalTimeoutConstant;
   timeouts.WriteTotalTimeoutMultiplier =WriteTotalTimeoutMultiplier;
   
   ready = SetCommTimeouts(handle, &timeouts);
   
   if (FALSE == ready)
   {
#ifdef DEBUG
      printf("Error: SetCommTimeouts returned a FALSE.\n");
#endif
      CloseHandle(handle);
      return 2;
   }

   return 0;
}



int serial_port::write_char(unsigned char data)
{
	unsigned long bytes_written;

   if (WriteFile(handle, &data, 1, &bytes_written, NULL) != FALSE)
   {
      if (bytes_written == 1)
      {
         return 0;
      }
      else
      {
#ifdef DEBUG
         printf("Error: WriteFile(1) reports an unexpected number of bytes written: %ld .\n", bytes_written);
#endif
      }
   }
   else
   {
#ifdef DEBUG
      check_and_report_misc_error();
      printf("Error: WriteFile(1) returned a FALSE.\n");
#endif
   }
   
   return 1;
}



int serial_port::read_char(unsigned char &data)
{
 unsigned char rx;
 unsigned long bytes_read = 0;

   if (ReadFile(handle, &rx, 1, &bytes_read, 0) != FALSE)
   {
      if (bytes_read == 1)
      {
         data = rx;
         return 0;
      }
      else
      {
#ifdef DEBUG
         printf("Error: ReadFile(1) reports an unexpected number of bytes read: %ld .\n", bytes_read);
#endif
      }
   }
   else
   {
#ifdef DEBUG
      printf("Error: ReadFile(1) returned a FALSE.\n");
#endif
   }
			  
	return 1;
}



int serial_port::write(unsigned char* data, unsigned long len)
{
 unsigned long bytes_written;

   if (WriteFile(handle, data, len, &bytes_written, NULL) != FALSE)
   {
      // WriteFile returned OK...
      if (bytes_written != len)
      {
#ifdef DEBUG
         check_and_report_misc_error();
         printf("Warning: WriteFile() reports an unexpected number of bytes written:\n");
         printf(" %ld instead of %ld.\n", bytes_written, len);
         printf("This probably means a timeout.\n");
#endif
      }

      return bytes_written;
   }
   else
   {
#ifdef DEBUG
      check_and_report_misc_error();
      printf("Error: WriteFile() returned a FALSE.\n");
#endif
      return -1;
   }
}



int serial_port::write_string(char* data)
{
   if (write((unsigned char*)data, strlen(data)) == (int)strlen(data))
      return 0;
   else
      return 1;
}



int serial_port::read(unsigned char* data, unsigned long len)
{
 unsigned long bytes_read = 0;

   if (ReadFile(handle, data, len, &bytes_read, 0) != FALSE)
   {
      if (bytes_read != len)
      {
#ifdef DEBUG
         printf("Error: ReadFile() reports an unexpected number of bytes read:\n");
         printf(" %ld instead of %ld.\n", bytes_read, len);
#endif
      }

      return bytes_read;
   }
   else
   {
#ifdef DEBUG
      printf("Error: ReadFile() returned a FALSE.\n");
#endif
      return -1;
   }
}



int serial_port::read_line(char* data, unsigned long buf_len, unsigned long timeout)
{
 unsigned long counter;
 int retval = 2; // buffer full, no newline detected

   // Set timeouts - since we're reading one character at a time,
   // the only timeout that makes sense is the absolute total timeout
   // (inter-character timeouts do not make sense - think about it).
   // To the calling code outside of this function though, 
   // that timeout appears like the inter-character timeout...
   if (set_comm_timeouts(0, 0, timeout, 0, 0) != 0)
      return retval;
  
   for (counter = 0; counter < buf_len; counter++)
   {
      if (read_char(((unsigned char*)data)[counter]) != 0) // if read failed
      {
         retval = 1;
         break;
      }

      if (data[counter] == '\r')
      {
         data[counter] = 0; // turn the newline into a terminating zero
         retval = 0;
         break;
      }
   }

   return retval;
}



int serial_port::expect(char** candidates, unsigned long timeout)
{
 int retval = -2;
#define EXPECT_BUF_LEN 256
 char data[EXPECT_BUF_LEN + 1]; // the data buffer + a trailing zero, just in case
 int counter;

   // Set timeouts - since we're reading one character at a time,
   // the only timeout that makes sense is the absolute total timeout
   // (inter-character timeouts do not make sense - think about it).
   // To the calling code outside of this function though, 
   // that timeout appears like the inter-character timeout...
   if (set_comm_timeouts(0, 0, timeout, 0, 0) != 0)
      return retval;

   memset(data, 0, EXPECT_BUF_LEN + 1);
 
   for (counter = 0;; counter++)
   { 
      if (read_char(((unsigned char*)data)[counter]) != 0) // if read failed
      {
         retval = -1;
         break;
      }

      // now check all the strings against the buffer - if success, set retval and break
      for (int i=0; candidates[i] != NULL; i++)
      {
         if (strstr(data, candidates[i]) != NULL) // if a match is found 
         {
            retval = i;
            // cannot break here - would only break the inner for(;;) cycle
            goto done;
         }
      }
      
      if (counter == (EXPECT_BUF_LEN - 1))
      {
         memcpy(data, data+1, EXPECT_BUF_LEN); // shift the buffer
         counter--;
      }
   }

done: 

   return retval;
}


   
void serial_port::purge()
{
   PurgeComm(handle, PURGE_RXABORT | PURGE_RXCLEAR | PURGE_TXABORT | PURGE_TXCLEAR);
   return;
}



int serial_port::set_rts()
{
   return(escape_function(SETRTS));
}


   
int serial_port::clear_rts()
{
   return(escape_function(CLRRTS));
}


   
int serial_port::set_dtr()
{
   return(escape_function(SETDTR));
}

   

int serial_port::clear_dtr()
{
   return(escape_function(CLRDTR));
}
   


int serial_port::escape_function(unsigned long func)
{
   if (EscapeCommFunction(handle, func) == 0)
   {
#ifdef DEBUG
      check_and_report_misc_error();
      printf("Error: EscapeCommFunction() failed.\n");
#endif
      return(1);
   }
   else
      return(0);
}

